/*global define*/
define([
    'sandbox/dom',
    'sandbox/event',
    'sandbox/view',
    'underscore'
], function ($, vent, View, _) {

    'use strict';

    return View.extend({

        initialize : function () {
            this.hightlightEvent = 'highlight';
            this.lastHoveredEvent = 'hovered.last';
            this.dropTargetEvent = 'target:drop';
            this.lastDropped = null;
        },

        events: {
            mousemove: 'onMove',
            mouseleave: 'onLeave',
            mouseenter: 'onEnter',
            dragover: 'onDragOver',
            dragenter: 'onEnter',
            dragleave: 'onDragLeave',
            drop: 'onDrop'
        },

        clearHovered : function () {
            vent.trigger(this.lastHoveredEvent, null);
            this.lastHovered = null;
        },

        onMove : function (event, style) {
            if (this.lastHovered !== event.target) {
                this.hover(event.target);
                vent.trigger(this.lastHoveredEvent, event.target, style);
            }
        },

        onDragOver : function (event) {
            var allowDrop = _.some(event.originalEvent.dataTransfer.types, function (sel) {
                    return $(event.target).is(sel);
                }),
                style = 'forbidden';

            if (allowDrop) {
                style = 'droppable';
            }

            this.onMove(event, style);

            if (style === 'droppable') {
                return false;
            }

            return true;
        },

        hover : function (element) {
            this.lastHovered = element;
            if ($(element).is('[data-type]')) {
                vent.trigger('outline.actions:show');
            } else {
                vent.trigger('outline.actions:hide');
            }
        },

        onEnter : function () {
            this.clearHovered();
        },

        onLeave : function () {
            this.clearHovered();
        },

        onDragLeave : function () {
            this.clearHovered();
        },

        onDrop : function (event) {
            this.lastDropped = event.target;
            vent.trigger(this.dropTargetEvent, this.lastDropped);
        },

        // Returns the element where the last `drop` happened.
        getLastDropped : function () {
            return this.lastDropped;
        }
    });
});
