/*global define*/
define([
    'sandbox/event',
    'sandbox/dom',
    'underscore',
    'backbone.marionette',
    'app/views/mouse',
    'app/templates/views/outline/outline'
], function (vent, $, _, Marionette, Mouse, template) {

    'use strict';

    var mouse = new Mouse(),

        styles = {
            'default': 'outline-default',
            'editable': 'outline-editable',
            'forbidden': 'outline-forbidden',
            'droppable': 'outline-droppable'
        },

        stylesClasses = _.values(styles).join(' '),

        alignTo = function (lastHovered, style) {
            var that = this,
                $iframe = that.options.iframe,
                iframeBoundingRect = $iframe.get(0).getBoundingClientRect(),
                iframeBodyBoundingRect = $($iframe.get(0).contentDocument).find('body').get(0).getBoundingClientRect(),
                lastHoveredBoundingRect,
                //iframeScrollY = $($('iframe').get(0).contentWindow).scrollTop(),
                top,
                left_correct,
                width_correct,
                height_correct,
                top_correct,
                i = 0,
                isBody;

            if (!lastHovered) {
                that.hide(0);

                return;
            }

            isBody = 'body' === lastHovered.nodeName.toLowerCase();
            lastHoveredBoundingRect = isBody ? _.clone(iframeBoundingRect) : lastHovered.getBoundingClientRect();

            if (isBody) {
                lastHoveredBoundingRect.right = lastHoveredBoundingRect.right - lastHoveredBoundingRect.left;
                lastHoveredBoundingRect.left = 0;
                lastHoveredBoundingRect.bottom = lastHoveredBoundingRect.bottom - lastHoveredBoundingRect.top;
                lastHoveredBoundingRect.top = 0;
            }

            top = lastHoveredBoundingRect.top; //- iframeScrollY;
            left_correct = -Math.min(lastHoveredBoundingRect.left, i);
            top_correct = -Math.min(Math.max(lastHoveredBoundingRect.top, 0), i);
            width_correct = Math.min(Math.abs(iframeBodyBoundingRect.width - lastHoveredBoundingRect.right), i) - left_correct;
            height_correct = Math.min(Math.abs(iframeBoundingRect.bottom - lastHoveredBoundingRect.bottom), i) - top_correct;

            that.$el
                .removeClass(stylesClasses)
                .addClass(styles[style] || styles['default'])
                .css({
                    left: lastHoveredBoundingRect.left + left_correct,
                    top: top + top_correct,
                    width: lastHoveredBoundingRect.width + width_correct,
                    height: lastHoveredBoundingRect.height + height_correct,
                    opacity: 1
                });

        };

    return Marionette.ItemView.extend({

        options: {
            iframe: null
        },

        attributes: {
            'data-view': 'outline'
        },

        tagName: 'div',

        id: 'outline-guide',

        className: 'outline outline-guide',

        template: template,

        ui: {
            actionbox: '.editor-element-toolbox'
        },

        events: {
            'click .tool-remove': 'deleteContent'
        },

        initialize: function () {
            this.listenTo(vent, mouse.lastHoveredEvent, alignTo);
            this.listenTo(vent, mouse.highlightEvent, alignTo);
            this.listenTo(vent, 'outline.actions:show', this.showActions);
            this.listenTo(vent, 'outline.actions:hide', this.hideActions);
        },

        deleteContent: function () {
            vent.trigger('editor.content:remove', mouse.lastHovered);
            return false;
        },

        hide: function () {
            this.$el.css('opacity', 0);
        },

        showActions: function () {
            if (!this.$el.hasClass(styles.droppable) && !this.$el.hasClass(styles.forbidden)) {
                this.ui.actionbox.show();
            }
        },

        hideActions: function () {
            this.ui.actionbox.hide();
        }

    });

});
