/*global define, describe, beforeEach, afterEach, it, expect, sinon, $*/
define(['app/app', 'sandbox/event', 'app/views/widgetitem'], function (app, vent, Widgetitem) {

    'use strict';

    describe('Views :: Widgetitem.', function () {

        beforeEach(function () {
            var that = this;
            this.model = {
                toJSON: function () {
                    return {
                        type: 'test',
                        title: 'test widget',
                        id: 'add-test',
                        icon: 'test',
                        description: 'this is a test'
                    };
                }
            };
            this.lastDropped = $('<span>').data('id', '1');
            this.view = new Widgetitem({ model: this.model, app: app });
            this.view.render();
            this.$widget = this.view.$el.find('span');
            app.editor = {
                pageModel: {
                    id: '1',
                    addContent: function () { return; }
                },
                getLastDropped: function () {
                    return that.lastDropped;
                },
                savePage: sinon.spy()
            };
        });

        afterEach(function () {
            this.view.close();
        });

        it('Rendering should create a "span"', function () {
            expect(this.view.$el).toBe('li');
            expect(this.view.$el).toContain('span');
            expect(this.view.$el.find('span')).toBe(this.$widget);
            expect(this.$widget).toBeMatchedBy('#add-test');
            expect(this.$widget).toHaveAttr('title', 'this is a test');
            expect(this.$widget).toContain('.icon-test');
        });

        describe('Add Content', function () {

            beforeEach(function () {
                this.requestData = {};
                this.addContentSpy = sinon.spy();
            });

            afterEach(function () {
                this.addContentSpy = null;
            });

            it('should trigger an add content event', function () {
                vent.on('editor.content:add', this.addContentSpy);

                this.view.addContent(this.requestData);

                expect(this.addContentSpy).toHaveBeenCalledOnce();
                expect(this.addContentSpy).toHaveBeenCalledWith(this.requestData);
            });

        });

        describe('Events', function () {

            beforeEach(function () {
                this.dragEndEvent = $.Event('dragend');
                this.dragStartEvent = $.Event('dragstart');
                this.dragEndEvent.originalEvent = {
                    dataTransfer: {
                        dropEffect: 'copy'
                    }
                };
                this.action = sinon.spy(this.view, 'action');
                this.dragStartSpy = sinon.spy();
                this.view.$('#' + this.model.toJSON().id).on('dragstart', this.dragStartSpy);
            });

            afterEach(function () {
                this.dragEndEvent = null;
                this.action.restore();
            });

            it('should call addContent method from dragEnd', function () {
                this.view.$('#' + this.model.toJSON().id).trigger(this.dragEndEvent);
                expect(this.action).toHaveBeenCalled();
            });

            it('dragEnd should not call addContent', function () {
                this.dragEndEvent.originalEvent.dataTransfer.dropEffect = 'none';
                this.view.$('#' + this.model.toJSON().id).trigger(this.dragEndEvent);
                expect(this.action).not.toHaveBeenCalled();
            });

            it('call dragStart', function () {
                this.view.$('#' + this.model.toJSON().id).trigger(this.dragStartEvent);
                expect(this.dragStartSpy).toHaveBeenCalledOnce();
            });

            it('call onHover', function () {
                this.view.onHover();
            });

        });

    });
});
