/*global define, describe, beforeEach, afterEach, it, expect, sinon, window, $*/
define(['app/views/widgets/text/button'], function (TextButton) {

    'use strict';

    describe('Views :: TextButton.', function () {

        beforeEach(function () {
            this.model = {
                toJSON: function () {
                    return {
                        type: 'text',
                        title: 'Add Text',
                        id: 'add-text',
                        icon: 'font',
                        description: 'add a text'
                    };
                }
            };
            this.view = new TextButton({
                model: this.model
            });
            this.view.render();
        });

        afterEach(function () {
            this.view.close();
        });

        describe('action', function () {
            var returned;

            beforeEach(function () {
                this.promptStub = sinon.stub(window, 'prompt', function () {
                    return returned;
                });
                this.addContentStub = sinon.stub(this.view, 'addContent');
            });

            afterEach(function () {
                this.promptStub.restore();
                this.addContentStub.restore();
            });

            it('should add the correct content', function () {
                returned = 'test';
                this.view.action();
                expect(this.promptStub).toHaveBeenCalledOnce();
                expect(this.promptStub).toHaveReturned(returned);
                expect(this.addContentStub).toHaveBeenCalledOnce();
            });

            it('should cancel the call', function () {
                returned = '';
                this.view.action();
                expect(this.promptStub).toHaveBeenCalledOnce();
                expect(this.promptStub).toHaveReturned(returned);
                expect(this.addContentStub).not.toHaveBeenCalled();
            });

        });

        describe('Events', function () {

            beforeEach(function () {
                this.event = $.Event('dragstart');
                this.event.originalEvent = {
                    dataTransfer: {
                        setData: sinon.spy()
                    }
                };
                this.dragStartSpy = sinon.spy(this.view, 'dragStart');
                this.setDataSpy = this.event.originalEvent.dataTransfer.setData;
            });

            afterEach(function () {
                this.dragStartSpy.restore();
                this.event = null;
            });

            it('should call dragStart on dragstart event', function () {
                this.view.$('#' + this.model.toJSON().id).trigger(this.event);
                expect(this.setDataSpy).toHaveBeenCalledWith('div[data-view="column"]:not(:has([data-type]))', '');
                expect(this.setDataSpy).toHaveBeenCalledWith('html', '');
            });

        });

    });
});
